@@ -33,8 +33,8 @@ test_that("can write to db", {
 	skip_if_not(can_con(pg), "could not connect to postgis database")
 	expect_silent(suppressMessages(st_write(pts, pg, "sf_meuse__")))
 	expect_error(st_write(pts, pg, "sf_meuse__"), "exists")
-	expect_true(st_write(pts, pg, "sf_meuse__", overwrite = TRUE))
-	expect_true(st_write(pts, pg, "sf_meuse2__", binary = FALSE))
+	expect_silent(st_write(pts, pg, "sf_meuse__", overwrite = TRUE))
+	expect_silent(st_write(pts, pg, "sf_meuse2__", binary = FALSE))
 	expect_warning(z <- st_set_crs(pts, epsg_31370))
 	expect_message(st_write(z, pg, "sf_meuse3__"), "Inserted local crs")
 	expect_silent(st_write(z, pg, "sf_meuse3__", append = TRUE))
